//----------------------------------------------------------------------------------
// File:   CarPaint.fxh
// Author: Tristan Lorach
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

// Carpaint shader

//////////////////////////////////////////////////////////////////////////////////
// Tweakables
//////////////////////////////////////////////////////////////////////////////////
cbuffer cb2 : register(b2)
{
    float paintamp < float uimin = 0; float uimax = 2; >                = 0.75;
    float diffStrength < float uimin = 0; float uimax = 2; >            = 0.42;
    float specStrength < float uimin = 0; float uimax = 6; >            = 1.15;
    float SpecExpStrengh < float uimin = 0; float uimax = 2.0; >        = 0.7;
    float reflection  < float uimin = 0; float uimax = 1; >             = 0.3;
    // Fleck-specific
    float3 fleck_material_color < float uimin = 0; float uimax = 1; >   = {1,1,1};
    // saturate the specular color
    float  fleck_locality < float uimin = 0; float uimax = 20; >        = 2.0;
    float  fleck_shininess < float uimin = 0; float uimax = 10; >       = 6.0;
    float fleck_scale < float uimin = 0; float uimax = 0.5; >           = 0.75;
    float freq < float uimin = 0; float uimax = 5; >                    = 2.0;
    float clearcoat_smoothness < float uimin = 0; float uimax = 100; >  = 4;
}
//////////////////////////////////////////////////////////////////////////////////
// VS to PS
//////////////////////////////////////////////////////////////////////////////////
struct CarPaint_VSIn
{
    float3 position : position;
    float3 normal : normal;
    float2 texcoord : texcoord;
};
struct CarPaint_PSOut
{
    float4 color : SV_Target0;
    float4 depth : SV_Target1;
};

struct CarPaint_VSOut
{
    float4 HPosition     : SV_Position;
    float4 color         : color;
    float  depth         : texcoord0;
    float3 WorldNormal   : texcoord1;
    float3 WorldView     : texcoord2;
    float3 Pw            : texcoord3;
    float3 Po            : texcoord4;
    float2 TexCoord      : TEXCOORD5;
};


//////////////////////////////////////////////////////////////////////////////////
// TEXTURE TEXTURE TEXTURE TEXTURE TEXTURE TEXTURE TEXTURE TEXTURE TEXTURE
//////////////////////////////////////////////////////////////////////////////////
Texture2D SceneTexture;
Texture2D paintMap < string file="Noise256_01.dds";>;
TextureCube envMap < string file="SkyCubemap.dds";>;
Texture3D noiseMap < string file="noise_MetalPaint.dds";>;
SamplerState samplerNoise
{
    AddressU = Mirror;
    AddressV = Mirror;
    Filter = Min_Mag_Mip_Linear;
};
SamplerState samplerMipMap
{
    AddressU = Mirror;
    AddressV = Mirror;
    Filter = Min_Mag_Mip_Linear;
};
SamplerState samplerLinear
{
    AddressU = Mirror;
    AddressV = Mirror;
    Filter = Min_Mag_Mip_Linear;
};
//////////////////////////////////////////////////////////////////////////////////
// FLECKS FLECKS FLECKS FLECKS FLECKS FLECKS FLECKS FLECKS FLECKS FLECKS FLECKS
//////////////////////////////////////////////////////////////////////////////////
float3 compute_flecks(inout float3 Nn, float3 Po, float3 Pw, float3 reflVect)
{
    // Scale the obj_coords to make things more sparkly
    float4 noise_coords = float4(Po * fleck_scale, 0);

    //      1a) Look up 4 noise values from the noiseMap, sampling at different frequencies (powers of freq)
    //          Hint: use float3 f3tex3D(texobj tex, float3 texcoords);
    float3 fleck_normal0 = noiseMap.SampleLevel(samplerNoise, noise_coords, 0).rgb;
    float3 fleck_normal1 = noiseMap.SampleLevel(samplerNoise, noise_coords * freq, 0).rgb;
    float3 fleck_normal2 = noiseMap.SampleLevel(samplerNoise, noise_coords * pow(freq, 2), 0).rgb;
    float3 fleck_normal3 = noiseMap.SampleLevel(samplerNoise, noise_coords * pow(freq, 3), 0).rgb;

    //      1b) Hack a specular component calculation - instead of (R dot L)^s, use (N dot E)^s
    //          This doesn't take the light position into account, but the normals are random so nobody
    //          will notice!  You will want to reverse the normal if it's pointing away from the eye.
    float  fleck_intensity = pow(saturate(abs(fleck_normal0.z)), fleck_shininess) + 
              pow(saturate(abs(fleck_normal1.z)), fleck_shininess) + 
              pow(saturate(abs(fleck_normal2.z)), fleck_shininess) + 
              pow(saturate(abs(fleck_normal3.z)), fleck_shininess);

    //      1c) Calculate specular component using this normal, just like you did above, with ONE EXCEPTION
    //          Raise r_dot_l from above by the fleck_locality constant and modulate into the computation
    //          This will restrict the sparkles to the vicinity of the specular highlights.
  
    float fleck_intensity_sum = 0.0;
    float3 L = normalize(lightPos - Pw);
    float n_dot_l = dot(Nn, L);
    float self_shadow = ( n_dot_l > 0) ? 1 : 0;
    float r_dot_l = self_shadow * saturate(dot(reflVect, -L));
    float locality_term = pow(r_dot_l, fleck_locality);
    fleck_intensity_sum = fleck_intensity * locality_term * self_shadow;
    //      1e) Finally, modulate with fleck's material color
    float3 fleck_color = fleck_intensity_sum * fleck_material_color;

    // PART  2  Add noise octaves together to form smooth value noise, use to slightly perturb
    //                       reflection vector to simulate slight irregularities on car surface
    //                       Hint: use the fleck_normal lookups from above
    //      2a) Synthesize a multi-frequency noise function by appropriately weighting each of the noise
    //           freqencies with diminishing weights (1/2, 1/4, 1/8, 1/16...)
    float3 noise_vector = .5 * fleck_normal0 + .25 * fleck_normal1 + 
    .125 * fleck_normal2 + .0625 * fleck_normal3;

    //      2b) Use this vector to perturb N very slightly
    Nn = normalize(noise_vector + clearcoat_smoothness * Nn);
    return fleck_color;
}
//////////////////////////////////////////////////////////////////////////////////
// VERTEX SHADER
//////////////////////////////////////////////////////////////////////////////////
CarPaint_VSOut carpaintVS(CarPaint_VSIn IN) {
    CarPaint_VSOut OUT;
    float3 N = mul(IN.normal, worldIT).xyz;
    float4 Po = float4(IN.position.xyz,1.0);    // object coordinates
    OUT.Po = IN.position.xyz;
    OUT.HPosition = mul(Po, worldViewProj);    // screen clipspace coords
    OUT.depth = OUT.HPosition.z;// MAYBE WRONG: FAR_CLIP - (OUT.HPosition.z + NEAR_CLIP);
    float3 Pw = mul(Po, world).xyz;        // world coordinates
    OUT.Pw = Pw;
    float4 viewPos = viewI[3];//mul(float4(0,0,0,1), viewI);
    OUT.WorldView = normalize(viewPos.xyz - Pw);    // obj coords;
    OUT.TexCoord = IN.texcoord.xy;
    OUT.WorldNormal = N;

    return OUT;
}


//////////////////////////////////////////////////////////////////////////////////
// PIXEL SHADERS PIXEL SHADERS PIXEL SHADERS PIXEL SHADERS PIXEL SHADERS
//////////////////////////////////////////////////////////////////////////////////
CarPaint_PSOut carpaintPS(CarPaint_VSOut IN,
    uniform float paintamp,
    uniform float Kd,
    uniform float Ks,
    uniform float Shininess,
    uniform float Kr,
    uniform float3 AmbiColor,
    uniform float3 SurfColor
    )
{
    CarPaint_PSOut output;
    float3 specContrib;
    float3 diffContrib;
    float3 Nn = normalize(IN.WorldNormal);
    float3 Vn = normalize(IN.WorldView);

    float3 reflVect = reflect(Vn, Nn);
    float3 fleck_color;
    fleck_color = compute_flecks(Nn, IN.Po, IN.Pw, reflVect);

    float3 Ln = normalize(lightPos - IN.Pw);
    float3 Hn = normalize(Vn + Ln);
    float4 litVec = lit(dot(Ln,Nn),dot(Hn,Nn),Shininess);
    diffContrib = (litVec.y * SpecExpStrengh) * lightColor;
    specContrib = (litVec.z * diffContrib) * lightColorSpec;
    // new reflection using perturbed normal
    reflVect = reflect(Vn, Nn);

    float vdn = dot(Vn,Nn);
    float3 reflColor = Kr * envMap.Sample(samplerLinear, reflVect).xyz;;
    // add, incorporating ambient light term
    float3 painttex = (paintamp.xxx * SceneTexture.Sample(samplerMipMap, IN.TexCoord).xyz) +
                      SurfColor + fleck_color;
    specContrib *= paintMap.Sample(samplerMipMap,IN.TexCoord).xyz;
    float3 result = (painttex*((Kd*diffContrib+AmbiColor))) + (Ks*specContrib + reflColor);
 
    output.color = float4(result.xyz,1.0);
    output.depth = IN.depth;
    return output;
}
//////////////////////////////////////////////////////////////////////////////////
// TECHNIQUES TECHNIQUES TECHNIQUES TECHNIQUES TECHNIQUES TECHNIQUES TECHNIQUES
//////////////////////////////////////////////////////////////////////////////////
technique10 CarPaint
{
    pass ColorNDepth
    {
        SetBlendState(blendOFF, float4(1.0, 1.0, 1.0, 1.0) ,0xffffffff);
        SetRasterizerState( RStateMSAA );
        SetDepthStencilState( depthEnabled, 0 );
        SetVertexShader( CompileShader( vs_4_0, carpaintVS() ) );
        SetGeometryShader(NULL);
        SetPixelShader( CompileShader( ps_4_0, carpaintPS(
            paintamp,
            diffStrength,specStrength,Shininess,
            reflection,
            ambiLightColor,surfColor) ) );
    }
}
