//----------------------------------------------------------------------------------
// File:   Sparkles.cpp
// Author: Tristan Lorach
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma warning (disable: 4995) // remove deprecated functions warning...

#include "DXUT.h"
#include "DXUTmisc.h"
#include "DXUTcamera.h"
#include "DXUTgui.h"
#include "DXUTsettingsDlg.h"
#include "SDKmisc.h"
#include "sdkmesh_old.h"
#include "resource.h"

#include "NVUTSkybox.h"

#include <fstream>

// new class to make RenderSubset public instead of protected
class CDXUTMesh10Ex : public CDXUTMesh10
{
public:
    void RenderSubset( ID3D10EffectTechnique *pTechnique, 
                       UINT pass, 
                       ID3D10EffectShaderResourceVariable* ptxDiffuse,
                       ID3D10EffectVectorVariable* pvDiffuse, 
                       ID3D10EffectVectorVariable* pvSpecular, 
                       DWORD dwSubset )
    {
        CDXUTMesh10::RenderSubset( pTechnique, pass, ptxDiffuse, pvDiffuse, pvSpecular, dwSubset );
    }
    void RenderInstancedSubset( ID3D10Device *pDev, 
                           ID3D10EffectTechnique *pTechnique, UINT pass,
                           UINT uiInstanceCount, 
                           ID3D10EffectShaderResourceVariable* ptxDiffuse,
                           ID3D10EffectVectorVariable* pvDiffuse, 
                           ID3D10EffectVectorVariable* pvSpecular,
                           DWORD dwSubset)
    {
        // Draw
        if( m_pMesh10 )
        {
            SetResources( ptxDiffuse, pvDiffuse, pvSpecular, dwSubset );
            pTechnique->GetPassByIndex( pass )->Apply( 0 );
            m_pMesh10->DrawSubsetInstanced( dwSubset, uiInstanceCount, 0);
        }
    }
};

CDXUTDialogResourceManager          g_DialogResourceManager;// manager for shared resources of dialogs
CD3DSettingsDlg                     g_D3DSettingsDlg;       // Device settings dialog
CDXUTDialog                         g_HUD;                  // manages the 3D UI
CDXUTDialog                         g_SampleUI;             // dialog for sample specific controls

ID3DX10Font*                        g_pFont = NULL;         // Font for drawing text
ID3DX10Sprite*                      g_pSprite = NULL;       // Sprite for batching text drawing

CDXUTMesh10Ex                       g_Mesh;
ID3D10EffectShaderResourceVariable* g_pSceneTextureShaderVariable = NULL;

ID3D10Texture2D*                    g_pRTTexture2DDepth     = NULL;
ID3D10RenderTargetView*             g_pRTViewDepth          = NULL;
ID3D10ShaderResourceView*           g_pTextureDepthView     = NULL;
ID3D10Texture2D*                    g_pRTTexture2DDepth_res = NULL;
ID3D10ShaderResourceView*           g_pTextureDepthView_res = NULL;
ID3D10InputLayout*                  g_pVertexLayout         = NULL;
ID3D10Effect*                       g_pEffect               = NULL;
int                                 g_curTechnique          = 0;

float                               g_ClearColor[]          = {0.0f, 0.0f, 0.4f, 1.0f};
float                               g_ClearColor2[]         = {1000.0, 1000.0, 1000.0, 1000.0};

bool                                g_Initialised           = false;
bool								g_InitialDeviceCreation = false;
bool                                g_bDrawSkyBox           = true;

// Num Instances of our object to draw
// each instance is dedicated to one sparkle layer in the triangle
// so it is also the max amount of sparkles we can get onto a triangle
int                                 g_numInstances          = 5;
float                               g_minSZ                 = 0.0f;
float                               g_maxSZ                 = 0.3f;
float                               g_LODScale              = 30.0f;
float                               g_Shininess             = 80.0f;
bool                                g_bNoPersp              = true;
bool                                g_Animate               = true;
float                               g_Alpha                 = 0.0f;
char                                g_bShowStats            = 1;
char                                g_bShowUI               = 1;
WCHAR                               g_FileName[200];

class CModelViewerCamera_Ex : public CModelViewerCamera
{ public:
    float GetNear() { return m_fNearPlane; }
    float GetFar()  { return m_fFarPlane;  } 
};
CModelViewerCamera_Ex    g_camera;

void CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext );
HRESULT CreateEffect(ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc);
HRESULT InitBackBufferDependentData(ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc);
bool InitInputGeometry(ID3D10Device* pd3dDevice);

/*----------------------------------------------
  --
  -- Message display
  --
  ----------------------------------------------*/
#define LOGMSG      PrintMessage // could be printf...
#define LOG_MSG     0
#define LOG_WARN    1
#define LOG_ERR     2
void PrintMessage(int level, LPCWSTR fmt, ...)
{
    static WCHAR dest[400];
    LPCWSTR *ppc = (LPCWSTR*)_ADDRESSOF(fmt);
    wvsprintf(dest, fmt, (va_list)&(ppc[1]));
    if(level == LOG_ERR) MessageBox(NULL, dest, L"Error", MB_OK|MB_ICONERROR);
    OutputDebugString(L"(SAMPLE) : ");
    if(level == LOG_WARN) OutputDebugString(L"WARNING: ");
    OutputDebugString(dest);
    OutputDebugString(L"\n");
}
/*----------------------------------------------
  --
  -- UI IDs
  --
  ----------------------------------------------*/
#define IDC_TOGGLEFULLSCREEN          1
#define IDC_TOGGLEREF                 2
#define IDC_CHANGEDEVICE              3
#define IDC_MAXSZ                     4
#define IDC_MINSZ                     5
#define IDC_LODSCALE                  6
#define IDC_SHININESS                 7
#define IDC_NOPERSP                   8
#define IDC_SKYBOX                    9
#define IDC_ANIMATE                   10
#define IDC_LOADMODEL                 11
#define IDC_RESETVALS                 12
#define IDC_MAXSPRITES                13
/*----------------------------------------------
  --
  --
  --
  ----------------------------------------------*/
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{    
    switch( nControlID )
    {
        case IDC_TOGGLEFULLSCREEN: DXUTToggleFullScreen(); break;
        case IDC_TOGGLEREF:        DXUTToggleREF(); break;
        case IDC_CHANGEDEVICE:     g_D3DSettingsDlg.SetActive( !g_D3DSettingsDlg.IsActive() ); break;
        case IDC_MAXSZ:
        {
            WCHAR sz[100];
            g_maxSZ = (float) (g_SampleUI.GetSlider( IDC_MAXSZ )->GetValue()* 0.001f);
            StringCchPrintf( sz, 100, L"MaxSz : %0.2f", g_maxSZ); 
            g_SampleUI.GetStatic( IDC_MAXSZ )->SetText( sz );
            g_pEffect->GetVariableByName("maxSZ")->AsScalar()->SetFloat(g_maxSZ);
            break;
        }
        case IDC_MINSZ:
        {
            WCHAR sz[100];
            g_minSZ = (float) (g_SampleUI.GetSlider( IDC_MINSZ )->GetValue()* 0.001f);
            StringCchPrintf( sz, 100, L"MinSz : %0.2f", g_minSZ); 
            g_SampleUI.GetStatic( IDC_MINSZ )->SetText( sz );
            g_pEffect->GetVariableByName("minSZ")->AsScalar()->SetFloat(g_minSZ);
            break;
        }
        case IDC_LODSCALE:
        {
            WCHAR sz[100];
            g_LODScale = (float) (g_SampleUI.GetSlider( IDC_LODSCALE )->GetValue()* 0.1f);
            StringCchPrintf( sz, 100, L"LODScale : %0.2f", g_LODScale); 
            g_SampleUI.GetStatic( IDC_LODSCALE )->SetText( sz );
            g_pEffect->GetVariableByName("LODScale")->AsScalar()->SetFloat(g_LODScale);
            break;
        }
        case IDC_MAXSPRITES:
        {
            WCHAR sz[100];
            g_numInstances = g_SampleUI.GetSlider( IDC_MAXSPRITES )->GetValue();
            StringCchPrintf( sz, 100, L"MaxSprites : %d", g_numInstances); 
            g_SampleUI.GetStatic( IDC_MAXSPRITES )->SetText( sz );
            g_pEffect->GetVariableByName("g_maxSprites")->AsScalar()->SetInt(g_numInstances);
            break;
        }
        case IDC_SHININESS:
        {
            WCHAR sz[100];
            g_Shininess = (float) (g_SampleUI.GetSlider( IDC_SHININESS )->GetValue()* 0.1f);
            StringCchPrintf( sz, 100, L"Shininess : %0.2f", g_Shininess); 
            g_SampleUI.GetStatic( IDC_SHININESS )->SetText( sz );
            g_pEffect->GetVariableByName("Shininess")->AsScalar()->SetFloat(g_Shininess);
            break;
        }
        case IDC_NOPERSP:
        {
            g_bNoPersp = g_SampleUI.GetCheckBox( IDC_NOPERSP )->GetChecked();
            g_pEffect->GetVariableByName("bNoPersp")->AsScalar()->SetInt(g_bNoPersp);
            break;
        }
        case IDC_SKYBOX:
        {
            g_bDrawSkyBox = g_SampleUI.GetCheckBox( IDC_SKYBOX )->GetChecked();
            break;
        }
        case IDC_ANIMATE:
        {
            g_Animate = g_SampleUI.GetCheckBox( IDC_ANIMATE )->GetChecked();
            break;
        }
        case IDC_RESETVALS:
        {
            WCHAR sz[100];
            g_maxSZ = 0.3f;
            g_SampleUI.GetSlider( IDC_MAXSZ )->SetValue((int)(1000.0f*g_maxSZ));
            StringCchPrintf( sz, 100, L"MaxSz : %0.2f", g_maxSZ); 
            g_SampleUI.GetStatic( IDC_MAXSZ )->SetText( sz );
            g_pEffect->GetVariableByName("maxSZ")->AsScalar()->SetFloat(g_maxSZ);
            g_minSZ = 0.0f;
            g_SampleUI.GetSlider( IDC_MINSZ )->SetValue((int)(1000.0*g_minSZ));
            StringCchPrintf( sz, 100, L"MinSz : %0.2f", g_minSZ); 
            g_SampleUI.GetStatic( IDC_MINSZ )->SetText( sz );
            g_pEffect->GetVariableByName("minSZ")->AsScalar()->SetFloat(g_minSZ);
            g_LODScale = 30.0f;
            g_SampleUI.GetSlider( IDC_LODSCALE )->SetValue((int)(10.0f*g_LODScale));
            StringCchPrintf( sz, 100, L"LODScale : %0.2f", g_LODScale); 
            g_SampleUI.GetStatic( IDC_LODSCALE )->SetText( sz );
            g_pEffect->GetVariableByName("LODScale")->AsScalar()->SetFloat(g_LODScale);
            g_Shininess = 80.0f;
            g_SampleUI.GetSlider( IDC_SHININESS )->SetValue((int)(10.0*g_Shininess));
            StringCchPrintf( sz, 100, L"Shininess : %0.2f", g_Shininess); 
            g_SampleUI.GetStatic( IDC_SHININESS )->SetText( sz );
            g_pEffect->GetVariableByName("Shininess")->AsScalar()->SetFloat(g_Shininess);
            g_bNoPersp = true;
            g_SampleUI.GetCheckBox( IDC_NOPERSP )->SetChecked(g_bNoPersp);
            g_pEffect->GetVariableByName("bNoPersp")->AsScalar()->SetInt(g_bNoPersp);
            D3DXVECTOR3 vFromPt   = D3DXVECTOR3(1.0f, 1.3f, -6.4f);
            D3DXVECTOR3 vLookatPt = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
            g_camera.SetViewParams( &vFromPt, &vLookatPt);
            D3DXVECTOR3 vModelPt = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
            g_camera.SetModelCenter(vModelPt);
            break;
        }
        case IDC_LOADMODEL:
        {
            OPENFILENAME info;
            ZeroMemory(&info, sizeof (OPENFILENAME));
            info.lStructSize = sizeof (OPENFILENAME);
            info.hwndOwner = DXUTGetHWND();
            info.lpstrFilter = L"X file meshes (.x)\0*.x\0\0";
            info.nFilterIndex = 0;
            info.lpstrFile = g_FileName;
            info.nMaxFile = 256;
            info.lpstrTitle = L"Open";
            info.nMaxFileTitle = sizeof(info.lpstrTitle);
            info.dwReserved = OFN_EXPLORER;

            if (GetOpenFileName(&info))
            {
                StringCchPrintf(g_FileName, 199, L"%s", info.lpstrFile); 
                InitInputGeometry(DXUTGetD3D10Device());
            }
            break;
        }
    }
}

/*----------------------------------------------
    --
    -- Misc helpers
    --
    ----------------------------------------------*/
namespace Tools 
{
    WCHAR                      g_path[MAX_PATH+1];
    char                       g_pathb[MAX_PATH+1];
    inline const char * PathNameAsBS(const char * filename)
    {
        HRESULT hr;
        WCHAR wfname[MAX_PATH];
        if(!filename)
            return NULL;
        mbstowcs_s(NULL, wfname, MAX_PATH, filename, MAX_PATH);
        hr = ::DXUTFindDXSDKMediaFileCch(Tools::g_path, MAX_PATH, wfname);
        if(FAILED(hr))
        {
            LOGMSG(LOG_ERR,L"couldn't find %S. Maybe wrong path...", filename);
            return NULL;
        }
        wcstombs_s(NULL, Tools::g_pathb, MAX_PATH, Tools::g_path, MAX_PATH);
        return Tools::g_pathb;
    }
    inline WCHAR * PathNameAsWS(LPWSTR filename)
    {
        HRESULT hr;
        if(!filename)
            return NULL;
        hr = ::DXUTFindDXSDKMediaFileCch(g_path, MAX_PATH, filename);
        if(FAILED(hr))
        {
            LOGMSG(LOG_ERR,L"couldn't find %S. Maybe wrong path...", filename);
            return NULL;
        }
        return g_path;
    }
    //--------------------------------------------------------------------------------------
    // GetPassDesc
    //--------------------------------------------------------------------------------------
    D3D10_PASS_DESC g_tmpPassDesc;
    D3D10_PASS_DESC * GetPassDesc(ID3D10Effect *pEffect, LPCSTR technique, LPCSTR pass)
    {
        ID3D10EffectTechnique * m_Tech = NULL;
        if(HIWORD(technique))
        m_Tech = pEffect->GetTechniqueByName(technique);
        else
        m_Tech = pEffect->GetTechniqueByIndex((UINT)LOWORD(technique));
        if(HIWORD(pass))
        {
            if ( FAILED( m_Tech->GetPassByName(pass)->GetDesc(&Tools::g_tmpPassDesc) ) )
            {
                LOGMSG(LOG_ERR, L"Failed getting description\n" );
                return NULL;
            }
        } else
        {
            if ( FAILED( m_Tech->GetPassByIndex((UINT)LOWORD(pass))->GetDesc(&Tools::g_tmpPassDesc) ) )
            {
                LOGMSG(LOG_ERR, L"Failed getting description\n" );
                return NULL;
            }
        }
        return &Tools::g_tmpPassDesc;
    }
    /*----------------------------------------------
      --
      --
      --
      ----------------------------------------------*/
    HRESULT CreateTexture2DFromFile(LPCSTR texfile, ID3D10Texture2D **ppTex, ID3D10ShaderResourceView **ppView, ID3D10Device* pd3dDevice)
    {
        HRESULT hr = S_OK;
        ID3D10Texture2D *pTex;
        ID3D10ShaderResourceView *pView;
        ID3D10Resource *pTexture;
        D3DX10_IMAGE_LOAD_INFO LoadInfo;
        D3DX10_IMAGE_INFO      SrcInfo;
        D3DX10GetImageInfoFromFileA(Tools::PathNameAsBS(texfile), NULL, &SrcInfo, &hr);

        LoadInfo.Width          = SrcInfo.Width;
        LoadInfo.Height         = SrcInfo.Height;
        LoadInfo.Depth          = SrcInfo.Depth;
        LoadInfo.FirstMipLevel  = 0;
        LoadInfo.MipLevels      = SrcInfo.MipLevels;
        LoadInfo.Usage          = D3D10_USAGE_DEFAULT;
        LoadInfo.BindFlags      = D3D10_BIND_SHADER_RESOURCE;
        LoadInfo.CpuAccessFlags = 0;
        LoadInfo.MiscFlags      = SrcInfo.MiscFlags;
        LoadInfo.Format         = SrcInfo.Format;
        LoadInfo.Filter         = D3DX10_FILTER_LINEAR;
        LoadInfo.MipFilter      = D3DX10_FILTER_LINEAR;
        LoadInfo.pSrcInfo       = &SrcInfo;

        D3DX10CreateTextureFromFileA( pd3dDevice, Tools::PathNameAsBS(texfile), &LoadInfo, NULL, &pTexture, &hr);
        if(FAILED(hr))
          return hr;
        D3D10_RESOURCE_DIMENSION d;
        pTexture->GetType(&d);
        hr = pTexture->QueryInterface(__uuidof(ID3D10Texture2D), (void**)&pTex);
        pTexture->Release();
        if(FAILED(hr))
        {
            LOGMSG(LOG_ERR,L"Error in QueryInterface(ID3D10Texture2D) for %S", Tools::PathNameAsBS(texfile));
            return hr;
        }
        D3D10_SHADER_RESOURCE_VIEW_DESC viewDesc;
        viewDesc.Format = LoadInfo.Format;
        viewDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
        viewDesc.Texture2D.MostDetailedMip = LoadInfo.FirstMipLevel;
        viewDesc.Texture2D.MipLevels = LoadInfo.MipLevels;
        if ( FAILED( pd3dDevice->CreateShaderResourceView( pTexture, &viewDesc, &pView) ) )
        {
            LOGMSG(LOG_ERR,L"Error in CreateShaderResourceView for %S", Tools::PathNameAsBS(texfile));
            if(ppTex) *ppTex = pTex;
            else pTex->Release();
            return hr;
        }
      if(ppTex) *ppTex = pTex;
      else pTex->Release();
      if(ppView) *ppView = pView;
      else pView->Release();
      return hr;
    }
    HRESULT CreateTexture3DFromFile(LPCSTR texfile, ID3D10Texture3D **ppTex, ID3D10ShaderResourceView **ppView, ID3D10Device* pd3dDevice)
    {
        HRESULT hr = S_OK;
        ID3D10Texture3D *pTex;
        ID3D10ShaderResourceView *pView;

        ID3D10Resource *pTexture;
        D3DX10_IMAGE_LOAD_INFO LoadInfo;
        D3DX10_IMAGE_INFO      SrcInfo;
        D3DX10GetImageInfoFromFileA(Tools::PathNameAsBS(texfile), NULL, &SrcInfo, &hr);

        LoadInfo.Width          = SrcInfo.Width;
        LoadInfo.Height         = SrcInfo.Height;
        LoadInfo.Depth          = SrcInfo.Depth;
        LoadInfo.FirstMipLevel  = 0;
        LoadInfo.MipLevels      = SrcInfo.MipLevels;
        LoadInfo.Usage          = D3D10_USAGE_DEFAULT;
        LoadInfo.BindFlags      = D3D10_BIND_SHADER_RESOURCE;
        LoadInfo.CpuAccessFlags = 0;
        LoadInfo.MiscFlags      = SrcInfo.MiscFlags;
        LoadInfo.Format         = SrcInfo.Format;
        LoadInfo.Filter         = D3DX10_FILTER_LINEAR;
        LoadInfo.MipFilter      = D3DX10_FILTER_LINEAR;
        LoadInfo.pSrcInfo       = &SrcInfo;

        D3DX10CreateTextureFromFileA( pd3dDevice, Tools::PathNameAsBS(texfile), &LoadInfo, NULL, &pTexture, &hr);
        if(FAILED(hr))
          return hr;
        D3D10_RESOURCE_DIMENSION d;
        pTexture->GetType(&d);
        hr = pTexture->QueryInterface(__uuidof(ID3D10Texture3D), (void**)&pTex);
        pTexture->Release();
        if(FAILED(hr))
        {
            LOGMSG(LOG_ERR,L"Error in QueryInterface(ID3D10Texture3D) for %S", texfile);
            return hr;
        }
        D3D10_SHADER_RESOURCE_VIEW_DESC viewDesc;
        viewDesc.Format = LoadInfo.Format;
        viewDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE3D;
        viewDesc.Texture2D.MostDetailedMip = LoadInfo.FirstMipLevel;
        viewDesc.Texture2D.MipLevels = LoadInfo.MipLevels;
        if ( FAILED( pd3dDevice->CreateShaderResourceView( pTexture, &viewDesc, &pView) ) )
        {
            LOGMSG(LOG_ERR,L"Error in CreateShaderResourceView for %S", texfile);
            if(ppTex) *ppTex = pTex;
            else pTex->Release();
            return hr;
        }

        LOGMSG(LOG_MSG, L"Created texture and view for %S", texfile);
        if(ppTex) *ppTex = pTex;
        else pTex->Release();
        if(ppView) *ppView = pView;
        else pView->Release();
        return S_OK;
    }
    HRESULT CreateCubeTextureFromFile(LPCSTR texfile, ID3D10Texture2D **ppTex, ID3D10ShaderResourceView **ppView, ID3D10Device* pd3dDevice)
    {
      HRESULT hr = S_OK;
        ID3D10Texture2D *pTex;
        ID3D10ShaderResourceView *pView;
        ID3D10Resource *pTexture;
        D3DX10_IMAGE_LOAD_INFO LoadInfo;
        D3DX10_IMAGE_INFO      SrcInfo;
        D3DX10GetImageInfoFromFileA(Tools::PathNameAsBS(texfile), NULL, &SrcInfo, &hr);

        LoadInfo.Width          = SrcInfo.Width;
        LoadInfo.Height         = SrcInfo.Height;
        LoadInfo.Depth          = SrcInfo.Depth;
        LoadInfo.FirstMipLevel  = 0;
        LoadInfo.MipLevels      = SrcInfo.MipLevels;
        LoadInfo.Usage          = D3D10_USAGE_DEFAULT;
        LoadInfo.BindFlags      = D3D10_BIND_SHADER_RESOURCE;
        LoadInfo.CpuAccessFlags = 0;
        LoadInfo.MiscFlags      = SrcInfo.MiscFlags;
        LoadInfo.Format         = SrcInfo.Format;
        LoadInfo.Filter         = D3DX10_FILTER_LINEAR;
        LoadInfo.MipFilter      = D3DX10_FILTER_LINEAR;
        LoadInfo.pSrcInfo       = &SrcInfo;

        D3DX10CreateTextureFromFileA( pd3dDevice, Tools::PathNameAsBS(texfile), &LoadInfo, NULL, &pTexture, &hr);
        if(FAILED(hr))
          return hr;
        D3D10_RESOURCE_DIMENSION d;
        pTexture->GetType(&d);
        hr = pTexture->QueryInterface(__uuidof(ID3D10Texture2D), (void**)&pTex);
        pTexture->Release();
        if(FAILED(hr))
        {
            LOGMSG(LOG_ERR,L"Error in QueryInterface(ID3D10Texture2D) for %S", texfile);
            return hr;
        }
        D3D10_SHADER_RESOURCE_VIEW_DESC viewDesc;
        viewDesc.Format = LoadInfo.Format;
        viewDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURECUBE;
        viewDesc.Texture2D.MostDetailedMip = LoadInfo.FirstMipLevel;
        viewDesc.Texture2D.MipLevels = LoadInfo.MipLevels;
        if ( FAILED( pd3dDevice->CreateShaderResourceView( pTexture, &viewDesc, &pView) ) )
        {
            LOGMSG(LOG_ERR,L"Error in CreateShaderResourceView for %S", texfile);
            if(ppTex) *ppTex = pTex;
            else pTex->Release();
            return hr;
        }
      LOGMSG(LOG_MSG, L"Created texture and view for %S", texfile);
      if(ppTex) *ppTex = pTex;
      else pTex->Release();
      if(ppView) *ppView = pView;
      else pView->Release();
      return S_OK;
    }

    //--------------------------------------------------------------------------------------
    // Load texture that we found in the effect : semantic "name"
    //--------------------------------------------------------------------------------------
    HRESULT LoadEffectTextures(ID3D10Effect* pEffect, ID3D10Device* pd3dDevice)
    {
        HRESULT h     = S_OK;
        HRESULT hrRet = S_OK;
        for(UINT i=0, hr = S_OK; !FAILED(hr); i++)
        {
            ID3D10EffectVariable *v = pEffect->GetVariableByIndex(i);
            if(v == NULL)
                break;
            D3D10_EFFECT_TYPE_DESC d;
            hr = v->GetType()->GetDesc(&d);
            ID3D10EffectVariable *f = v->GetAnnotationByName("file");
            if(!f)
            {
                LOGMSG(LOG_WARN, L"no 'file' annotation for the texture");
                continue;
            }
            //
            // Yeah sorry : we take a mbs, search with wcs and is it as mbs...
            //
            const char * pFname = NULL;
            ID3D10ShaderResourceView * texView = NULL;
            if(d.Type == D3D10_SVT_TEXTURE2D)
            {
                if( FAILED( h=f->AsString()->GetString(&pFname) ) )
                {
                    hrRet = h;
                    //LOGMSG(LOG_WARN, L"cannot get the name of the 'file' semantic" );
                }
                else if( FAILED( h=CreateTexture2DFromFile(pFname, NULL, &texView, pd3dDevice) ) )
                {
                    hrRet = h;
                    LOGMSG(LOG_ERR, L"failed to load texture %S", pFname );
                }
                else if( texView && FAILED( h=v->AsShaderResource()->SetResource(texView) ) )
                {
                    hrRet = h;
                    LOGMSG(LOG_ERR, L"failed to bind texture_face with DDS image" );
                }
                SAFE_RELEASE(texView);
            }
            else if(d.Type == D3D10_SVT_TEXTURECUBE)
            {
                if( FAILED( h=f->AsString()->GetString(&pFname) ) )
                {
                    hrRet = h;
                    //LOGMSG(LOG_WARN, L"cannot get the name of the 'file' semantic" );
                }
                else if( FAILED( h= CreateCubeTextureFromFile(pFname, NULL, &texView, pd3dDevice) ) )
                {
                    hrRet = h;
                    LOGMSG(LOG_ERR, L"failed to load texture %S", pFname );
                }
                else if( FAILED( h=v->AsShaderResource()->SetResource(texView) ) )
                {
                    hrRet = h;
                    LOGMSG(LOG_ERR, L"failed to bind texture_face with DDS image" );
                }
                SAFE_RELEASE(texView);
            }
            else if(d.Type == D3D10_SVT_TEXTURE3D)
            {
                if( FAILED( h=f->AsString()->GetString(&pFname) ) )
                {
                    hrRet = h;
                    //LOGMSG(LOG_WARN, L"cannot get the name of the 'file' semantic" );
                }
                else if( FAILED( h= CreateTexture3DFromFile(pFname, NULL, &texView, pd3dDevice) ) )
                {
                    hrRet = h;
                    LOGMSG(LOG_ERR, L"failed to load texture %S", pFname );
                }
                else if( FAILED( h=v->AsShaderResource()->SetResource(texView) ) )
                {
                    hrRet = h;
                    LOGMSG(LOG_ERR, L"failed to bind texture_face with DDS image" );
                }
                SAFE_RELEASE(texView);
            }
        }
        return hrRet;
    }
}// Tools

NVUTSkybox g_Skybox;    // skybox

// Environment map for the skybox
ID3D10Texture2D * g_EnvMap = NULL;                    
ID3D10ShaderResourceView * g_EnvMapSRV = NULL;


/*----------------------------------------------
  --
  --
  --
  ----------------------------------------------*/
bool InitInputGeometry(ID3D10Device* pd3dDevice)
{
    g_Mesh.Destroy();
    //
    // Define vertex data layout
    //
    const D3D10_INPUT_ELEMENT_DESC layout[] =
    {
        { "position", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "normal",   0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "texcoord",    0, DXGI_FORMAT_R32G32_FLOAT, 0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    };
    UINT numElementsScene = sizeof(layout)/sizeof(layout[0]);
    //
    // Simple DXUT Mesh
    //
    WCHAR stra[MAX_PATH];
    if(FAILED(DXUTFindDXSDKMediaFileCch(stra, MAX_PATH, g_FileName)))
    {
        LOGMSG(LOG_ERR, L"failed to load the model\n" );
        return false;
    }
    g_Mesh.Create( pd3dDevice,stra, (D3D10_INPUT_ELEMENT_DESC*)layout, numElementsScene );
    if(g_pEffect)
        g_pSceneTextureShaderVariable = g_pEffect->GetVariableByName("SceneTexture")->AsShaderResource();
    //
    // get Signature and create the layout
    //
    if(g_pEffect)
    {
        D3D10_PASS_DESC passDesc;
        ID3D10EffectTechnique * pTech = NULL;
        pTech = g_pEffect->GetTechniqueByIndex(g_curTechnique);
        if( FAILED( pTech->GetPassByIndex(0)->GetDesc(&passDesc) ) )
        {
            LOGMSG(LOG_ERR, L"GetDesc() failed.\n" );
            return false;
        }
        SAFE_RELEASE( g_pVertexLayout);
        if(  FAILED(  pd3dDevice->CreateInputLayout( layout, numElementsScene, passDesc.pIAInputSignature, passDesc.IAInputSignatureSize, &g_pVertexLayout ) ) )
        {
            LOGMSG(LOG_ERR, L"CreateElementLayout() failed.\n" );
            return false;
        }
    }
    return true;
}



//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    return true;
}


//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
#ifdef _DEBUG
	pDeviceSettings->d3d10.CreateFlags |= D3D10_CREATE_DEVICE_DEBUG;
#endif
    return true;
}

void CreateUI(ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc)
{
    WCHAR sz[100];
    //
    // UI
    //
    g_D3DSettingsDlg.GetDialogControl()->RemoveAllControls();
    g_D3DSettingsDlg.Init( &g_DialogResourceManager );
    g_HUD.RemoveAllControls();
    g_SampleUI.RemoveAllControls();
    g_HUD.Init( &g_DialogResourceManager );
    g_SampleUI.Init( &g_DialogResourceManager );

    g_HUD.SetCallback( OnGUIEvent ); 
    int iY = 10; 
    int iX = pBackBufferSurfaceDesc->Width - 130;
    g_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen (F6)", iX, iY, 125, 22, VK_F6 );
    g_HUD.AddButton( IDC_TOGGLEREF, L"Toggle REF (F3)", iX, iY += 24, 125, 22, VK_F3 );
    g_HUD.AddButton( IDC_CHANGEDEVICE, L"Change device (F2)", iX, iY += 24, 125, 22, VK_F2 );
    g_HUD.AddButton( IDC_LOADMODEL, L"Load Model (F4)", iX, iY += 24, 125, 22, VK_F4 );
    g_HUD.AddButton( IDC_RESETVALS, L"Reset (F5)", iX, iY += 24, 125, 22, VK_F5 );

    g_SampleUI.SetCallback( OnGUIEvent ); 

    StringCchPrintf( sz, 100, L"MinSz : %0.2f", g_minSZ); 
    g_SampleUI.AddStatic( IDC_MINSZ, sz, iX, iY += 24, 125, 22 );
    g_SampleUI.AddSlider( IDC_MINSZ, iX+15, iY += 24, 100, 22, 0, 1000, (int)(g_minSZ*1000) );

    StringCchPrintf( sz, 100, L"MaxSz : %0.2f", g_maxSZ); 
    g_SampleUI.AddStatic( IDC_MAXSZ, sz, iX, iY += 24, 125, 22 );
    g_SampleUI.AddSlider( IDC_MAXSZ, iX+15, iY += 24, 100, 22, 0, 4000, (int)(g_maxSZ*1000) );

    StringCchPrintf( sz, 100, L"LODScale : %0.2f", g_LODScale); 
    g_SampleUI.AddStatic( IDC_LODSCALE, sz, iX, iY += 24, 125, 22 );
    g_SampleUI.AddSlider( IDC_LODSCALE, iX+15, iY += 24, 100, 22, 0, 5000, (int)(g_LODScale*10) );

    StringCchPrintf( sz, 100, L"Sprites per tri : %d", g_numInstances); 
    g_SampleUI.AddStatic( IDC_MAXSPRITES, sz, iX, iY += 24, 125, 22 );
    g_SampleUI.AddSlider( IDC_MAXSPRITES, iX+15, iY += 24, 100, 22, 0, 100, g_numInstances );

    StringCchPrintf( sz, 100, L"Shininess : %0.2f", g_Shininess); 
    g_SampleUI.AddStatic( IDC_SHININESS, sz, iX, iY += 24, 125, 22 );
    g_SampleUI.AddSlider( IDC_SHININESS, iX+15, iY += 24, 100, 22, 0, 1280, (int)(g_Shininess*10) );

    g_SampleUI.AddCheckBox( IDC_NOPERSP, L"NO PERSP", iX+15, iY += 24, 125, 22, g_bNoPersp );
    g_SampleUI.AddCheckBox( IDC_SKYBOX, L"SkyBox", iX+15, iY += 24, 125, 22, g_bDrawSkyBox );
    g_SampleUI.AddCheckBox( IDC_ANIMATE, L"Animate", iX+15, iY += 24, 125, 22, g_Animate );
}
//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;
    g_Initialised = true;
    if(FAILED(CreateEffect(pd3dDevice, pBackBufferSurfaceDesc)))
        g_Initialised = false;
    if(g_Initialised && (!InitInputGeometry(pd3dDevice)))
        g_Initialised = false;
    if(g_Initialised)
        Tools::LoadEffectTextures(g_pEffect, pd3dDevice);

    CreateUI(pd3dDevice, pBackBufferSurfaceDesc);

    V_RETURN( g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( g_D3DSettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, 
                                OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, 
                                L"Arial", &g_pFont ) );
    V_RETURN( D3DX10CreateSprite( pd3dDevice, 512, &g_pSprite ) );

    V_RETURN( g_Skybox.OnCreateDevice( pd3dDevice ) );
    
    // Load envmap for the skybox
    hr = Tools::CreateCubeTextureFromFile("Skycubemap.dds", &g_EnvMap, &g_EnvMapSRV, pd3dDevice);
    if(FAILED(hr))
    {
       LOGMSG(LOG_ERR, L"Failed to load Skybox map");
       return hr;
    }

    g_Skybox.SetTexture(g_EnvMapSRV);
    return S_OK;
}
//--------------------------------------------------------------------------------------
// init window-size dependent stuff
//--------------------------------------------------------------------------------------
HRESULT InitBackBufferDependentData(ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc)
{
    if(!g_Initialised)
        return E_FAIL;
    //
    //Screen ration infos needed by the shader
    //
    UINT nVP = 1; //Beuark
    D3D10_VIEWPORT vp;
    pd3dDevice->RSGetViewports(&nVP, &vp);
    float v4[4] = { vp.MinDepth, vp.MaxDepth, (float)pBackBufferSurfaceDesc->Width/(float)pBackBufferSurfaceDesc->Height, 0 };
    if( FAILED( g_pEffect->GetVariableByName("near_far_scrratio")->AsVector()->SetFloatVector(v4) ) )
    {
        LOGMSG(LOG_WARN, L"update of near_far_scrratio failed" );
    }
    v4[0] = (float)pBackBufferSurfaceDesc->Width;
    v4[1] = (float)pBackBufferSurfaceDesc->Height;
    if( FAILED( g_pEffect->GetVariableByName("DepthRTSz")->AsVector()->SetFloatVector(v4) ) )
    {
        LOGMSG(LOG_WARN, L"update of DepthRTSz failed" );
    }
    //
    // Second render target for depth
    //
    ID3D10Texture2D           *pRTTexture2D;
    DXUTGetDXGISwapChain()->GetBuffer(0, __uuidof(*pRTTexture2D), reinterpret_cast<void**> ( &pRTTexture2D ) );
    D3D10_TEXTURE2D_DESC rtTexDesc;
    pRTTexture2D->GetDesc( &rtTexDesc ); // take some data from the basic RT
    rtTexDesc.Format = DXGI_FORMAT_R32_FLOAT;
    rtTexDesc.MipLevels = 1;
    rtTexDesc.ArraySize = 1;
    rtTexDesc.SampleDesc.Count = pBackBufferSurfaceDesc->SampleDesc.Count;
    rtTexDesc.SampleDesc.Quality = pBackBufferSurfaceDesc->SampleDesc.Quality;
    rtTexDesc.BindFlags = D3D10_BIND_SHADER_RESOURCE|D3D10_BIND_RENDER_TARGET;
    rtTexDesc.Usage = D3D10_USAGE_DEFAULT;
    rtTexDesc.CPUAccessFlags = 0;
    rtTexDesc.MiscFlags = 0;
    SAFE_RELEASE(pRTTexture2D);
    SAFE_RELEASE(g_pRTTexture2DDepth);
    if( FAILED( pd3dDevice->CreateTexture2D(&rtTexDesc, NULL, &g_pRTTexture2DDepth) ) )
    {
        LOGMSG(LOG_ERR, L"CreateTexture2D() failed.");
        return E_FAIL;
    }
    SAFE_RELEASE(g_pRTViewDepth);
    if( FAILED( pd3dDevice->CreateRenderTargetView(g_pRTTexture2DDepth, NULL, &g_pRTViewDepth) ) )
    {
        LOGMSG(LOG_ERR, L"CreateRenderTargetView() failed.");
        return E_FAIL;
    }
    //
    // Create the View of the texture pRTTexture2DDepth
    //
    D3D10_SHADER_RESOURCE_VIEW_DESC viewDesc;
    viewDesc.Format = DXGI_FORMAT_R32_FLOAT;
    if(pBackBufferSurfaceDesc->SampleDesc.Count > 1)
    {
        viewDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2DMS;
    } else {
        viewDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
        viewDesc.Texture2D.MostDetailedMip = 0;
        viewDesc.Texture2D.MipLevels = 1;
    }
    SAFE_RELEASE(g_pTextureDepthView);
    if ( FAILED( pd3dDevice->CreateShaderResourceView( g_pRTTexture2DDepth, &viewDesc, &g_pTextureDepthView) ) )
    {
        LOGMSG(LOG_ERR,L"Error in CreateShaderResourceView for Depth Render target");
        return E_FAIL;
    }
    LOGMSG(LOG_MSG, L"Created texture and view for pRTTexture2DDepth");
    //
    // in case of MSAA we want to expose a solution where we do a 'resolve' of the MSAA surface
    // here we declare the resolved surface to be used as a texture
    //
    SAFE_RELEASE(g_pRTTexture2DDepth_res);
    SAFE_RELEASE(g_pTextureDepthView_res);
    if(rtTexDesc.SampleDesc.Count > 1)
    {
        LOGMSG(LOG_MSG,L"MSAA to %d. Creating a render target to resolve MSAA RT...", rtTexDesc.SampleDesc.Count);
        rtTexDesc.Format = DXGI_FORMAT_R32_FLOAT;
        rtTexDesc.MipLevels = 1;
        rtTexDesc.ArraySize = 1;
        rtTexDesc.SampleDesc.Count = 1;
        rtTexDesc.SampleDesc.Quality = 0;
        rtTexDesc.BindFlags = D3D10_BIND_SHADER_RESOURCE|D3D10_BIND_RENDER_TARGET; // TODO: remove D3D10_BIND_RENDER_TARGET
        rtTexDesc.Usage = D3D10_USAGE_DEFAULT;
        rtTexDesc.CPUAccessFlags = 0;
        rtTexDesc.MiscFlags = 0;
        if( FAILED( pd3dDevice->CreateTexture2D(&rtTexDesc, NULL, &g_pRTTexture2DDepth_res) ) )
        {
            LOGMSG(LOG_ERR, L"CreateTexture2D() failed.");
            return E_FAIL;
        }
        //
        // Create the shader resource View of the resolved texture
        //
        D3D10_SHADER_RESOURCE_VIEW_DESC viewDesc;
        viewDesc.Format = DXGI_FORMAT_R32_FLOAT;
        viewDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
        viewDesc.Texture2D.MostDetailedMip = 0;
        viewDesc.Texture2D.MipLevels = 1;
        if ( FAILED( pd3dDevice->CreateShaderResourceView( g_pRTTexture2DDepth_res, &viewDesc, &g_pTextureDepthView_res) ) )
        {
            LOGMSG(LOG_ERR,L"Error in CreateShaderResourceView for Depth Render target");
            return E_FAIL;
        }
        LOGMSG(LOG_MSG, L"Created texture and view to resolve pRTTexture2DDepth");
    }
    return S_OK;
}
//--------------------------------------------------------------------------------------
// create the effect
//--------------------------------------------------------------------------------------
HRESULT CreateEffect(ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc)
{
    ID3D10Blob *pErrors = NULL;
    HRESULT hr;
    WCHAR path[MAX_PATH];

    hr = DXUTFindDXSDKMediaFileCch(path, MAX_PATH, L"Sparkles.fx");
    SAFE_RELEASE(g_pEffect);
    if( !FAILED(hr) )
    {
        D3DX10CreateEffectFromFile(path, NULL, NULL, "fx_4_0", D3D10_SHADER_NO_PRESHADER, 0, pd3dDevice, NULL, NULL, &g_pEffect, &pErrors, &hr);
    }

    if( FAILED( hr ) )
    {
        LOGMSG(LOG_ERR, L"Failed creating Effect from file\n" );
        if(pErrors) {
            LPCSTR szErrors = (LPCSTR)pErrors->GetBufferPointer();
            MessageBoxA(NULL, szErrors, "Effect errors", MB_OK);
            pErrors->Release();
        }
        return hr;
    }
    else LOGMSG(LOG_MSG, L"Effect loaded" );

    D3D10_EFFECT_DESC effectdesc;
    g_pEffect->GetDesc(&effectdesc);
    for(unsigned int i=0; i<effectdesc.Techniques;i++)
    {
      ID3D10EffectTechnique *tech = g_pEffect->GetTechniqueByIndex(i);
      if(!tech)
        break;
      D3D10_TECHNIQUE_DESC desc;
      tech->GetDesc(&desc);
      if(!strcmp(desc.Name, "Sparkles_And_CarPaint"))
      {
          LOGMSG(LOG_MSG, L"setting SparklesCarPaint as the default technique" );
          g_curTechnique = i;
      }
    }

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr = S_OK;
    V_RETURN( g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBufferSurfaceDesc ) );
    V_RETURN( g_D3DSettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBufferSurfaceDesc ) );
    //
    // Update Camera
    //
    float fAspectRatio = (float)pBufferSurfaceDesc->Width / (float)pBufferSurfaceDesc->Height;
    g_camera.SetProjParams( D3DX_PI/4, fAspectRatio, 0.1f, 100.0f );
    g_camera.SetWindow( pBufferSurfaceDesc->Width, pBufferSurfaceDesc->Height);

    //
    // Update things that are dependend on the window size
    //
    if(FAILED(InitBackBufferDependentData(pd3dDevice, pBufferSurfaceDesc)))
        g_Initialised = false;

    CreateUI(pd3dDevice, pBufferSurfaceDesc);

    g_Skybox.OnResizedSwapChain( pd3dDevice, pBufferSurfaceDesc );

    return hr;
}


//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
    g_camera.FrameMove(fElapsedTime);
}


//--------------------------------------------------------------------------------------
// Render the help and statistics text
//--------------------------------------------------------------------------------------
void RenderText()
{
    CDXUTTextHelper txtHelper( g_pFont, g_pSprite, 15 );

    txtHelper.Begin();
    txtHelper.SetInsertionPos( 2, 0 );
    txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
    txtHelper.DrawTextLine( DXUTGetFrameStats(true) );
    txtHelper.DrawTextLine( DXUTGetDeviceStats() );
    txtHelper.DrawTextLine( L"F9/F10 : Toggle UI/Stats" );
    txtHelper.End();
}
//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
    HRESULT hr;
    // If the settings dialog is being shown, then
    // render it instead of rendering the app's scene
    if( g_D3DSettingsDlg.IsActive() )
    {
        g_D3DSettingsDlg.OnRender( fElapsedTime );
        return;
    }
    if(!g_Initialised)
    {
        DXUTPause(true, true);
        return;
    }
    if(g_Animate)
    {
        g_Alpha += fElapsedTime * 0.1f;//0.002f;
    }

    D3DXMATRIX mProj = *g_camera.GetProjMatrix();
    D3DXMATRIX mView = *g_camera.GetViewMatrix();
    D3DXMATRIX mWorld;
    D3DXMATRIX MRotY;
    D3DXMatrixRotationY(&MRotY, g_Alpha);
    D3DXMatrixMultiply( &mWorld, &MRotY, g_camera.GetWorldMatrix() );

    // Calculate the matrix world*view*proj
    D3DXMATRIX mViewI, mViewIT, mViewProj, mViewProjI;
    D3DXMATRIX mWorldI, mWorldIT;
    D3DXMATRIX mWorldView, mWorldViewI, mWorldViewIT;
    D3DXMATRIX mWorldViewProj, mWorldViewProjI;
    D3DXMatrixInverse(&mViewI, NULL, &mView);
    D3DXMatrixTranspose(&mViewIT, &mViewI);
    D3DXMatrixMultiply( &mViewProj, &mView, &mProj );
    D3DXMatrixInverse(&mViewProjI, NULL, &mViewProj);
    D3DXMatrixInverse(&mWorldI, NULL, &mWorld);
    D3DXMatrixTranspose(&mWorldIT, &mWorldI);
    D3DXMatrixMultiply( &mWorldView, &mWorld, &mView );
    D3DXMatrixInverse(&mWorldViewI, NULL, &mWorldView);
    D3DXMatrixTranspose(&mWorldViewIT, &mWorldViewI);
    D3DXMatrixMultiply( &mWorldViewProj, &mWorldView, &mProj );
    D3DXMatrixInverse(&mWorldViewProjI, NULL, &mWorldViewProj);

    g_pEffect->GetVariableByName("worldViewProj")->AsMatrix()->SetMatrix(mWorldViewProj);
    g_pEffect->GetVariableByName("viewProj")->AsMatrix()->SetMatrix(mViewProj);
    g_pEffect->GetVariableByName("worldView")->AsMatrix()->SetMatrix(mWorldView);
    g_pEffect->GetVariableByName("worldViewIT")->AsMatrix()->SetMatrix(mWorldViewIT);
    g_pEffect->GetVariableByName("proj")->AsMatrix()->SetMatrix(mProj);
    g_pEffect->GetVariableByName("worldIT")->AsMatrix()->SetMatrix(mWorldIT);
    g_pEffect->GetVariableByName("world")->AsMatrix()->SetMatrix(mWorld);
    g_pEffect->GetVariableByName("viewIT")->AsMatrix()->SetMatrix(mViewIT);
    g_pEffect->GetVariableByName("viewI")->AsMatrix()->SetMatrix(mViewI);
    g_pEffect->GetVariableByName("view")->AsMatrix()->SetMatrix(mView);
    //
    // Set the render targets
    // - RT for color
    // - RT for depth
    // Note: this is somehow bad to use the back buffer color here : AA mode can be a problem
    //
    ID3D10ShaderResourceView *rv = NULL;
    ID3D10RenderTargetView *pRTs[] = {DXUTGetD3D10RenderTargetView(), g_pRTViewDepth};
    pd3dDevice->OMSetRenderTargets(2, pRTs, DXUTGetD3D10DepthStencilView());
    //
    //Clear the rendertarget
    //
    pd3dDevice->ClearDepthStencilView(DXUTGetD3D10DepthStencilView(), D3D10_CLEAR_DEPTH, 1.0f, 0);
    pd3dDevice->ClearRenderTargetView( DXUTGetD3D10RenderTargetView(), g_ClearColor);
    g_ClearColor2[0] = g_camera.GetFar();
    pd3dDevice->ClearRenderTargetView( g_pRTViewDepth, g_ClearColor2);
    //
    // Sky box
    //
    if(g_bDrawSkyBox)
        g_Skybox.OnFrameRender(mViewProj); // render skybox
    //
    // Render the Object
    //
    ID3D10Buffer *pNullBuffers[] = { NULL };
    UINT offsets[2] = {0, 0};
    pd3dDevice->SOSetTargets( 1, pNullBuffers, offsets );

    pd3dDevice->IASetInputLayout( g_pVertexLayout );
    g_Mesh.RenderSubset( g_pEffect->GetTechniqueByIndex(g_curTechnique), 0, g_pSceneTextureShaderVariable, NULL, NULL, 0 );
    //
    // back to a single render target
    //
    ID3D10RenderTargetView *pRTView[2] = {DXUTGetD3D10RenderTargetView(), NULL}; // Due to a Bug, we must do this, right ?
    pd3dDevice->OMSetRenderTargets(2, pRTView, DXUTGetD3D10DepthStencilView());

    //
    // Render the Sparkles on top of the previously rendered scene
    //
    ID3D10EffectPass *pass = g_pEffect->GetTechniqueByIndex(g_curTechnique)->GetPassByIndex(1);
    if(pass)
    {
        // Use either
        // 1- a resolved version of the R32Float Render target.
        // 2- basic mode when no MSAA set
        if(g_pTextureDepthView_res) // we have MSAA because we create this intermediate
        {
            pd3dDevice->ResolveSubresource(g_pRTTexture2DDepth_res, 0,g_pRTTexture2DDepth, 0, DXGI_FORMAT_R32_FLOAT);
            hr = g_pEffect->GetVariableByName("texture_depth")->AsShaderResource()->SetResource(g_pTextureDepthView_res);
        } else { // g_pRTTexture2DDepth_res == 0 meaning that we are not in any MSAA mode
            hr = g_pEffect->GetVariableByName("texture_depth")->AsShaderResource()->SetResource(g_pTextureDepthView);
        }
        pd3dDevice->SOSetTargets( 0, pNullBuffers, offsets );

        pd3dDevice->IASetInputLayout( g_pVertexLayout );

        g_Mesh.RenderInstancedSubset(pd3dDevice, g_pEffect->GetTechniqueByIndex(g_curTechnique), 1, g_numInstances, NULL, NULL, NULL, 0);

        pd3dDevice->GSSetShaderResources(0, 1, &rv);
    }
    //-------------------------------------------------------------------------------------
    //HUD
    //-------------------------------------------------------------------------------------
    if(g_bShowStats)
    {
        DXUT_BeginPerfEvent( DXUT_PERFEVENTCOLOR, L"HUD / Stats" );
        RenderText();
        DXUT_EndPerfEvent();
    }
    if(g_bShowUI)
    {
        g_SampleUI.OnRender( fElapsedTime );
    }
    g_HUD.OnRender( fElapsedTime );
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10ReleasingSwapChain();

    g_Skybox.OnReleasingSwapChain();
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
    g_D3DSettingsDlg.OnD3D10DestroyDevice();
    g_D3DSettingsDlg.GetDialogControl()->RemoveAllControls();
    g_DialogResourceManager.OnD3D10DestroyDevice();
    SAFE_RELEASE( g_pRTTexture2DDepth_res);
    SAFE_RELEASE( g_pTextureDepthView_res);
    SAFE_RELEASE( g_pRTTexture2DDepth);
    SAFE_RELEASE( g_pRTViewDepth);
    SAFE_RELEASE( g_pTextureDepthView);
    SAFE_RELEASE( g_pVertexLayout);
    SAFE_RELEASE( g_pFont );
    SAFE_RELEASE( g_pSprite );
    SAFE_RELEASE( g_pEffect );
    SAFE_RELEASE( g_EnvMapSRV );
    SAFE_RELEASE( g_EnvMap );
    g_Skybox.OnDestroyDevice();
    g_Mesh.Destroy();
}


//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                          bool* pbNoFurtherProcessing, void* pUserContext )
{
    // Pass messages to dialog resource manager calls so GUI state is updated correctly
    *pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    // Pass messages to settings dialog if its active
    if( g_D3DSettingsDlg.IsActive() )
    {
        g_D3DSettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
        return 0;
    }

    // Give the dialogs a chance to handle the message first
    *pbNoFurtherProcessing = g_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;
    *pbNoFurtherProcessing = g_SampleUI.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    // Pass all remaining windows messages to camera so it can respond to user input
    g_camera.HandleMessages( hWnd, uMsg, wParam, lParam );
    return 0;
}


//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
  if(bKeyDown)
    switch(nChar)
    {
    case VK_F9:
        g_bShowStats ^= 1;
        break;
    case VK_F10:
        g_bShowUI ^= 1;
        break;
    case VK_ESCAPE:
      PostQuitMessage( 0 );
        break;
    }
}


//--------------------------------------------------------------------------------------
// Handle mouse button presses
//--------------------------------------------------------------------------------------
void CALLBACK OnMouse( bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown, 
                       bool bSideButton1Down, bool bSideButton2Down, int nMouseWheelDelta, 
                       int xPos, int yPos, void* pUserContext )
{
}


//--------------------------------------------------------------------------------------
// Call if device was removed.  Return true to find a new device, false to quit
//--------------------------------------------------------------------------------------
bool CALLBACK OnDeviceRemoved( void* pUserContext )
{
    return true;
}


//--------------------------------------------------------------------------------------
// Initialize everything and go into a render loop
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // DXUT will create and use the best device (either D3D9 or D3D10) 
    // that is available on the system depending on which D3D callbacks are set below

    // Set general DXUT callbacks
    DXUTSetCallbackFrameMove( OnFrameMove );
    DXUTSetCallbackKeyboard( OnKeyboard );
    DXUTSetCallbackMouse( OnMouse );
    DXUTSetCallbackMsgProc( MsgProc );
    DXUTSetCallbackDeviceChanging( ModifyDeviceSettings );
    DXUTSetCallbackDeviceRemoved( OnDeviceRemoved );

   // Set the D3D10 DXUT callbacks. Remove these sets if the app doesn't need to support D3D10
    DXUTSetCallbackD3D10DeviceAcceptable( IsD3D10DeviceAcceptable );
    DXUTSetCallbackD3D10DeviceCreated( OnD3D10CreateDevice );
    DXUTSetCallbackD3D10SwapChainResized( OnD3D10ResizedSwapChain );
    DXUTSetCallbackD3D10FrameRender( OnD3D10FrameRender );
    DXUTSetCallbackD3D10SwapChainReleasing( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed( OnD3D10DestroyDevice );

    DXUTSetMediaSearchPath(L"..\\Source\\Sparkles\\");
    StringCchPrintf( g_FileName, 100, L"cat.x"); 

    DXUTInit( true, true, NULL ); // Parse the command line, show msgboxes on error, no extra command line params
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
    DXUTCreateWindow( L"Sparkles", NULL, NULL, NULL, 0, 0);
	// See ModifyDeviceSettings.  We set this flag on first creation to avoid the warning message box.
	g_InitialDeviceCreation = true;
    DXUTCreateDevice( true, 800, 600 );  
	g_InitialDeviceCreation = false;

    //
    // Camera
    //
    D3DXVECTOR3 vFromPt   = D3DXVECTOR3(1.0f, 1.3f, -6.4f);
    D3DXVECTOR3 vLookatPt = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
    g_camera.SetViewParams( &vFromPt, &vLookatPt);
    D3DXVECTOR3 vModelPt = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
    g_camera.SetModelCenter(vModelPt);

    DXUTMainLoop(); // Enter into the DXUT render loop

    return DXUTGetExitCode();
}


